﻿using ProjectByldozerHard.Entities;

namespace ProjectByldozerHard.Drawing;

public class DrawningTrackedCar
{
    private InterDopRink? DrawingKatki;
    /// <summary>
    /// Класс-сущность
    /// </summary>
    public EntityTrackedCar? EntityTrackedCar { get; protected set; }

    /// <summary>
    /// Ширина окна
    /// </summary>
    private int _pictureWidth;

    /// <summary>
    /// Высота окна
    /// </summary>
    private int _pictureHeight;

    /// <summary>
    /// Левая координата прорисовки бульдозера
    /// </summary>
    protected int _startPosX;

    /// <summary>
    /// Верхняя кооридната прорисовки  бульдозера
    /// </summary>
    protected int _startPosY;

    /// <summary>
    /// Ширина прорисовки  бульдозера
    /// </summary>
    private readonly int _drawningByldozerWidth = 150;

    /// <summary>
    /// Высота прорисовки  бульдозера
    /// </summary>
    private readonly int _drawningByldozerHeight = 80;

    /// <summary>
	/// Координата X объекта
	/// </summary>
	public int GetPosX => _startPosX;

    /// <summary>
    /// Координата Y объекта
    /// </summary>
    public int GetPosY => _startPosY;

    /// <summary>
    /// Ширина объекта
    /// </summary>
    public int GetWidth => _drawningByldozerWidth;

    /// <summary>
    /// Высота объекта
    /// </summary>
    public int GetHeight => _drawningByldozerHeight;

    public DrawningTrackedCar(int speed, double weight, Color bodyColor, int width, int height, int numKatki, int numchoose)
    {
        if (width > _drawningByldozerWidth || height > _drawningByldozerHeight)
        {
            _pictureWidth = width;
            _pictureHeight = height;
            EntityTrackedCar = new EntityTrackedCar(speed, weight, bodyColor);
            int choose = numchoose % 3;
            switch (choose)
            {
                case 0:
                    DrawingKatki = new DrawingRinkSquare();
                    break;
                case 1:
                    DrawingKatki = new DrawingRinkTriagle();
                    break;
                case 2:
                    DrawingKatki = new DrawingRinkCircule();
                    break;
            }
            DrawingKatki.Properties = numKatki;
        }
    }


    /// <summary>
	/// Конструктор
	/// </summary>
	/// <param name="speed">Скорость</param>
	/// <param name="weight">Вес</param>
	/// <param name="bodyColor">Основной цвет</param>
    protected DrawningTrackedCar(int speed, double weight, Color bodyColor, int width, int height, int exWidth, int exHeight, int numKatki)
    {
        if (width > _drawningByldozerWidth || height > _drawningByldozerHeight)
        {
            _pictureWidth = width;
            _pictureHeight = height;
            _drawningByldozerWidth = exWidth;
            _drawningByldozerHeight = exHeight;
            EntityTrackedCar = new EntityTrackedCar(speed, weight, bodyColor);
        }
    }


    /// <summary>
    /// Установка границ поля
    /// </summary>
    /// <param name="width">Ширина поля</param>
    /// <param name="height">Высота поля</param>4ееее
    /// <returns>true - границы заданы, false - проверка не пройдена, нельзя разместить объект в этих размерах</returns>


    /// <summary>
    /// Установка позиции
    /// </summary>
    /// <param name="x">Координата X</param>
    /// <param name="y">Координата Y</param>
    public void SetPosition(int x, int y)
    {
        if (x < 0)
        {
            x = 0;
        }
        else if (x > _pictureWidth - _drawningByldozerWidth)
        {
            x = _pictureWidth - _drawningByldozerWidth;

        }

        if (y < 0)
        {
            y = 0;
        }
        else if (y > _pictureHeight - _drawningByldozerHeight)
        {
            y = _pictureHeight - _drawningByldozerHeight;
        }
        _startPosX = x;
        _startPosY = y;

    }


    public bool CanMove(DirectionType direction)
    {
        if (EntityTrackedCar == null)
        {
            return false;
        }
        return direction switch
        {
            //влево
            DirectionType.Left => _startPosX - EntityTrackedCar.Step > 0,
            //вверх
            DirectionType.Up => _startPosY - EntityTrackedCar.Step > 0,
            // вправо
            DirectionType.Right => _startPosX + _drawningByldozerWidth + EntityTrackedCar.Step <= _pictureWidth,
            //влево
            DirectionType.Down => _startPosY + _drawningByldozerHeight + EntityTrackedCar.Step <= _pictureHeight,
        };
    }


    /// <summary>
    /// Изменение направления перемещения
    /// </summary>
    /// <param name="direction">Направление</param>
    /// <returns>true - перемещение выполнено, false - перемещение невозможно</returns>
    public void MoveTransport(DirectionType direction)
    {
        if (!CanMove(direction) || EntityTrackedCar == null)
        {
            return;
        }

        switch (direction)
        {
            //влево
            case DirectionType.Left:
                _startPosX -= (int)EntityTrackedCar.Step;
                break;
            //вверх
            case DirectionType.Up:
                _startPosY -= (int)EntityTrackedCar.Step;
                break;
            // вправо
            case DirectionType.Right:
                _startPosX += (int)EntityTrackedCar.Step;
                break;
            //вниз
            case DirectionType.Down:
                _startPosY += (int)EntityTrackedCar.Step;
                break;
            default:
                break;
        }
    }

    /// <summary>
    /// Прорисовка объекта
    /// </summary>
    /// <param name="g"></param>
    public virtual void DrawTransport(Graphics g)
    {
        if (EntityTrackedCar == null)
        {
            return;
        }
        Pen pen = new(Color.Black);
        Brush bodybrush = new SolidBrush(EntityTrackedCar.BodyColor);

        //границы бульдозера 
        g.DrawRectangle(pen, _startPosX, _startPosY + 15, 100, 30);
        g.DrawRectangle(pen, _startPosX + 70, _startPosY, 30, 20);

        g.DrawArc(pen, _startPosX, _startPosY + 40, 30, 40, 130, 100);
        g.DrawArc(pen, _startPosX + 73, _startPosY + 45, 30, 40, 258, 140);

        g.DrawLine(pen, _startPosX + 5, _startPosY + 75, _startPosX + 100, _startPosY + 75);


        DrawingKatki.Draw(_startPosX, _startPosY, EntityTrackedCar.BodyColor, g);
        //заливка кабины и гусениц
        Brush br = new SolidBrush(Color.Pink);
        g.FillRectangle(br, _startPosX, _startPosY + 15, 100, 30);
        g.FillRectangle(br, _startPosX + 70, _startPosY, 30, 20);
       

    }


}
