﻿
using ProjectByldozerHard.Drawing;
using ProjectByldozerHard.MovementStrategy;

namespace ProjectByldozerHard;

public partial class FormByldozer : Form
{


    /// <summary>
    /// поле объект для прорисовки объекта
    /// </summary>
    private DrawningTrackedCar? _drawingByldozer;
    private AbstractStrategy? _abstractStrategy;

    public FormByldozer()
    {
        InitializeComponent();
        comboBoxStrategy.Items.Add("К центру");
        comboBoxStrategy.Items.Add("К краю");
    }
    private void Draw()
    {
        if (_drawingByldozer == null)
        {
            return;
        }

        Bitmap bmp = new(pictureBoxByldozer.Width, pictureBoxByldozer.Height);
        Graphics gr = Graphics.FromImage(bmp);
        _drawingByldozer.DrawTransport(gr);
        pictureBoxByldozer.Image = bmp;
    }

    /// <summary>
    /// обработка кнопки создать
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonCreat_Click(object sender, EventArgs e)
    {
        Random random = new();
        _drawingByldozer = new DrawningTrackedCar(random.Next(100, 300), random.Next(1000, 3000),
        Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)),
        pictureBoxByldozer.Width, pictureBoxByldozer.Height, random.Next(1, 4), random.Next(1, 4));
        _drawingByldozer.SetPosition(random.Next(10, 100), random.Next(10, 100));
        Draw();
    }

    /// <summary>
    /// перемещение объекта
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonMove_Click(object sender, EventArgs e)
    {
        
        if (_drawingByldozer == null)
        {
            return;
        }

        string name = ((Button)sender)?.Name ?? string.Empty;

        switch (name)
        {
            case "buttonUp":
                _drawingByldozer.MoveTransport(DirectionType.Up);
                break;
            case "buttonDown":
                _drawingByldozer.MoveTransport(DirectionType.Down);
                break;
            case "buttonLeft":
                _drawingByldozer.MoveTransport(DirectionType.Left);
                break;
            case "buttonRight":
                _drawingByldozer.MoveTransport(DirectionType.Right);
                break;
        }

        Draw();
    }

    private void ButtonStep_Click(object sender, EventArgs e)
    {
        if (_drawingByldozer == null)
        {
            return;
        }
        if (comboBoxStrategy.Enabled)
        {
            _abstractStrategy = comboBoxStrategy.SelectedIndex
            switch
            {
                0 => new MoveToCenter(),
                1 => new MoveToBorder(),
                _ => null,
            };
            if (_abstractStrategy == null)
            {
                return;
            }
            _abstractStrategy.SetData(new MoveableByldozer(_drawingByldozer), pictureBoxByldozer.Width, pictureBoxByldozer.Height);
            comboBoxStrategy.Enabled = false;
        }
        if (_abstractStrategy == null)
        {
            return;
        }
        _abstractStrategy.MakeStep();
        Draw();
        if (_abstractStrategy.GetStatus() == StrategyStatus.Finish)
        {
            comboBoxStrategy.Enabled = true;
            _abstractStrategy = null;
        }
    }

    private void ButtonCreateExKovsh_Click(object sender, EventArgs e)
    {
        Random random = new();
        _drawingByldozer = new DrawningByldozer(random.Next(100, 300), random.Next(1000, 3000),
        Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)),
        Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)),
        Convert.ToBoolean(random.Next(0, 2)),
        Convert.ToBoolean(random.Next(0, 2)),
        Convert.ToBoolean(random.Next(0, 2)),
        Convert.ToBoolean(random.Next(0, 2)),
        pictureBoxByldozer.Width, pictureBoxByldozer.Height, random.Next(1, 4), random.Next(1, 4));
        _drawingByldozer.SetPosition(random.Next(10, 100), random.Next(10, 100));
        Draw();
    }
}


